@@ -162,30 +162,46 @@ check_data_GRP_size <- function(x, GRP){
   }
 }
 
+# Alternate mixed method
+# GRP_duplicated <- function(GRP, all = FALSE){
+#   sizes <- GRP_group_sizes(GRP)
+#   group_id <- GRP_group_id(GRP)
+#   if (is.null(GRP[["order"]])){
+#     out <- (sizes > 1L)[group_id]
+#     if (!all){
+#       out[GRP_starts(GRP)] <- FALSE
+#     }
+#   } else {
+#     if (all){
+#       out <- (sizes > 1L)[group_id]
+#     } else {
+#       out <- row_id(GRP) > 1L
+#     }
+#   }
+#   out
+# }
+
 GRP_duplicated <- function(GRP, all = FALSE){
   sizes <- GRP_group_sizes(GRP)
   group_id <- GRP_group_id(GRP)
-  if (all){
-    out <- (sizes > 1L)[group_id]
-  } else {
-    out <- row_id(GRP) > 1L
+  out <- (sizes > 1L)[group_id]
+  if (!all){
+    out[GRP_starts(GRP)] <- FALSE
   }
   out
 }
-# Alternative that just returns dup indices
-# The commented-out code is simpler but which_val is more efficient
-# Especially with data that contains few duplicates
+# Alternate version
 # GRP_which_duplicated <- function(GRP, all = FALSE){
-#   which(GRP_duplicated(GRP, all))
+#   sizes <- GRP_group_sizes(GRP)
+#   group_id <- GRP_group_id(GRP)
+#   if (all){
+#     which((sizes > 1L)[group_id])
+#   } else {
+#     which_val(row_id(GRP), 1L, invert = TRUE)
+#   }
 # }
 GRP_which_duplicated <- function(GRP, all = FALSE){
-  sizes <- GRP_group_sizes(GRP)
-  group_id <- GRP_group_id(GRP)
-  if (all){
-    which((sizes > 1L)[group_id])
-  } else {
-    which_val(row_id(GRP), 1L, invert = TRUE)
-  }
+  which(GRP_duplicated(GRP, all))
 }
 sorted_group_starts <- function(group_sizes, init_loc = 1L){
   cpp_sorted_group_starts(as.integer(group_sizes), init_loc)
@@ -205,13 +221,13 @@ GRP_starts <- function(GRP, use.g.names = FALSE){
         out <- sorted_group_starts(GRP_sizes)
       }
       # For factors with 0 size, replace calculated group starts with 0
-      out[which_val(GRP_sizes, 0L)] <- 0L
+      out[cheapr::which_val(GRP_sizes, 0L)] <- 0L
     } else {
       o <- GRP_order(GRP)
       starts <- attr(o, "starts")
       if (collapse::anyv(GRP_sizes, 0L)){
         out <- integer(GRP_n_groups(GRP))
-        out[which_val(GRP_sizes, 0L, invert = TRUE)] <- o[starts]
+        out[cheapr::which_val(GRP_sizes, 0L, invert = TRUE)] <- o[starts]
       } else {
         out <- o[starts]
       }
@@ -232,7 +248,7 @@ GRP_ends <- function(GRP, use.g.names = FALSE,
   if (GRP_is_sorted(GRP)){
     out <- sorted_group_ends(GRP_sizes)
     # For factors with 0 size, replace 0 with NA
-    out[which_val(GRP_sizes, 0L)] <- 0L
+    out[cheapr::which_val(GRP_sizes, 0L)] <- 0L
   } else {
     if (is.null(loc)){
       loc <- GRP_loc(GRP, use.g.names = FALSE)
@@ -496,6 +512,10 @@ GRP.vctrs_rcrd <- function(X, ...){
 GRP.NULL <- function(X, ...){
   NULL
 }
+#' @exportS3Method collapse::GRP
+GRP.integer64 <- function(X, ...){
+  GRP2(cpp_int64_to_numeric(X), ...)
+}
 gsplit2 <- function(x = NULL, g = NULL, use.g.names = FALSE, ...){
   if (is.null(g)){
     if (is.null(x)){
